@@ -8,6 +8,7 @@ import sys
 import threading
 from contextlib import contextmanager
 
+import sqlalchemy
 from sqlalchemy import MetaData, create_engine, event, func, inspect
 from sqlalchemy.engine.url import make_url
 from sqlalchemy.ext.automap import automap_base, generate_relationship
@@ -15,18 +16,13 @@ from sqlalchemy.schema import conv
 from sqlalchemy.sql.expression import select
 from sqlalchemy.types import Text
 
-from . import VERSION
+from . import SQLALCHEMY_VERSION, VERSION
 from .configuration import DEFAULT_CONFIG
 from .models import BaseDeclarativeMeta, BaseModel
 from .query import BaseQuery, QueryProperty
 from .session import SessionProperty
-from .utils import (
-    aslist,
-    cached_property,
-    create_directory,
-    generate_valid_index_name,
-    to_unicode,
-)
+from .utils import (aslist, cached_property, create_directory,
+                    generate_valid_index_name, to_unicode)
 
 try:
     from easy_profile import SessionProfiler, StreamReporter
@@ -99,7 +95,8 @@ class Database(object):
             )
         else:
             db_cache_dir = os.path.join(self.uri.drivername, self.uri.database)
-        _cache_dir = os.path.join(self.global_cache_dir, VERSION, db_cache_dir)
+        cache_version = "{}-sa-{}".format(VERSION, sqlalchemy.__version__)
+        _cache_dir = os.path.join(self.global_cache_dir, cache_version, db_cache_dir)
         create_directory(_cache_dir)
         return _cache_dir
 
@@ -448,7 +445,7 @@ class EngineConnector(object):
                     )
                     options["connect_args"] = connect_args
                     if info.drivername == "mysql":
-                        info.query.setdefault("charset", "utf8")
+                        connect_args.setdefault("charset", "utf8")
                         options.setdefault("pool_size", 10)
                         options.setdefault("pool_recycle", 3600)
 
@@ -460,7 +457,11 @@ class EngineConnector(object):
                             pass
 
                     elif info.drivername == "postgresql":
-                        options.setdefault("use_batch_mode", True)
+                        if SQLALCHEMY_VERSION >= "1.4.0":
+                            options.setdefault("executemany_mode", "batch")
+                            options.setdefault("executemany_batch_page_size", 5000)
+                        else:
+                            options.setdefault("use_batch_mode", True)
 
                 elif info.drivername == "sqlite":
                     no_pool = options.get("pool_size") == 0
