@@ -1,10 +1,15 @@
 # -*- coding: utf-8 -*-
+from sqlalchemy import event
 from sqlalchemy.orm import Session, scoped_session, sessionmaker
 
+from . import SQLALCHEMY_VERSION
+from .query import _apply_backref_limit
 from .utils import merge_dicts
 
 
 class BaseSession(Session):
+    parsed_query = None
+
     def __init__(self, db, **options):
         self.db = db
         bind = options.pop("bind", None) or db.engine
@@ -16,6 +21,16 @@ class BaseSession(Session):
 
         Session.__init__(self, bind=bind, query_cls=query_cls, **session_options)
 
+        if SQLALCHEMY_VERSION >= "1.4.0":
+            event.listen(self, "do_orm_execute", self.receive_do_orm_execute)
+
+    def receive_do_orm_execute(self, orm_execute_state):
+        if orm_execute_state.is_select:
+            # Apply backref_limit to selectin queries
+            orm_execute_state.statement = _apply_backref_limit(
+                orm_execute_state.statement, self
+            )
+
 
 class SessionProperty(object):
 
